import random
import unittest

import numpy as np
import torch

from .utils import get_fi

random.seed(100)
np.random.seed(100)
torch.manual_seed(100)


class ProtectionTest(unittest.TestCase):
    def setUp(self) -> None:
        self.fi = get_fi()
        self.unprotected = self.fi.emat_attack(10, 1e-5)
        return super().setUp()

    def test_relu6(self):
        self.fi.relu6_protection()
        protected = self.fi.emat_attack(10, 1e-5)
        print(protected)
        print(self.unprotected)
        self.assertTrue(protected != self.unprotected)

    def test_zscoreDR(self):
        self.fi.outlierDR_protection()
        protected = self.fi.emat_attack(10, 1e-5)
        self.assertTrue(protected < self.unprotected)



import random

import numpy as np
import torch
from torchvision import models

from torchei import fault_model

random.seed(100)
np.random.seed(100)
torch.manual_seed(100)


def get_fi(name="squeeze") -> fault_model:
    model = models.mobilenet_v2(pretrained=True)
    valid = torch.load("./datasets/ilsvrc_valid8.pt")[:1]
    return fault_model(model, valid)


def check_range(x, l_bound=0.01, r_bound=0.9, rate=1) -> bool:
    return l_bound * rate < x < r_bound * rate



import random
import unittest
from time import sleep

import numpy as np
import torch

from .utils import check_range, get_fi

random.seed(100)
np.random.seed(100)
torch.manual_seed(100)


class ToolTest(unittest.TestCase):
    def setUp(self) -> None:
        self.fi = get_fi()
        return super().setUp()

    # sern出错了
    def test_sern(self):
        sern = self.fi.sern_calc(1000)
        self.assertTrue(check_range(sern[0], 0.02, 0.04))

    def test_time_decorator(self):
        @self.fi.time_decorator
        def hibernate(x):
            sleep(x)

        span = random.randrange(0, 5)
        hibernate(span)
        self.assertTrue(check_range(self.fi.time, span - 0.1, span + 0.1, 1000))

    def test_change_layer_filter(self):
        return 1

    def test_synthesis_error(self):
        self.fi.get_emat_single_func()
        self.assertTrue(check_range(self.fi.synthesis_error, 8, 12))

    def test_register_hook(self):
        self.fi.register_hook()
        self.assertEqual(len(self.fi.handles), len(self.fi.keys))

    def test_delimit(self):
        limit_points = self.fi.delimit()
        param_size = self.fi.get_param_size()
        self.assertAlmostEqual(10 ** (-limit_points[0]) * param_size, 100, delta=50)



import random
import unittest
from statistics import mean

import numpy as np
import torch

from .utils import check_range, get_fi

random.seed(100)
np.random.seed(100)
torch.manual_seed(100)


class AttackTest(unittest.TestCase):
    def setUp(self) -> None:
        self.fi = get_fi()
        return super().setUp()

    def test_emat(self):
        result = self.fi.emat_attack(10, 1e-5)
        self.assertTrue(check_range(result))

    def test_mc(self):
        result = self.fi.mc_attack(10, 1e-5)
        self.assertTrue(check_range(result))

    def test_layer_single_attack(self):
        result = self.fi.layer_single_attack(10)
        self.assertTrue(check_range(mean(result), 1 / 64 / 2, 0.1))
        attack_func = self.fi.get_emat_single_func()
        self.fi.layer_single_attack(10, attack_func)
        self.assertTrue(check_range(mean(result), 1 / 64 / 2, 0.1))

    def test_reliability_calc(self):
        fi = self.fi

        def f(_):
            return False

        def t(_):
            return True

        result = fi.emat_attack(
            10,
            1e-5,
            group_size=2,
            adaptive=True,
            adaptive_func=f,
        )
        self.assertIsInstance(result, float)

        # test adaptive
        result = fi.emat_attack(
            10,
            1e-5,
            kalman=True,
            verbose_return=True,
            group_size=2,
            init_size=4,
            adaptive=True,
            adaptive_func=f,
        )
        self.assertEqual(result[-1], 5)
        result = fi.emat_attack(
            10,
            1e-5,
            kalman=True,
            verbose_return=True,
            group_size=2,
            init_size=4,
            adaptive=True,
            adaptive_func=t,
        )
        self.assertEqual(result[-1], 1)

        # test verbose_return
        result = fi.emat_attack(
            10, 1e-5, kalman=True, verbose_return=True, group_size=2, init_size=4
        )
        self.assertIsInstance(result, tuple)
        result = fi.emat_attack(
            10,
            1e-5,
            kalman=True,
            verbose_return=False,
            group_size=2,
            init_size=4,
            adaptive=True,
        )
        self.assertIsInstance(result, float)
        result = fi.emat_attack(
            10,
            1e-5,
            kalman=False,
            verbose_return=True,
            group_size=2,
            init_size=4,
            adaptive=True,
        )
        self.assertIsInstance(result, tuple)

    def test_neuron_ei(self):
        result = self.fi.mc_attack(10, 1e-5, attack_type="neuron")
        self.assertTrue(check_range(result))
